=== modified file 'lib/canonical/launchpad/mailman/itests/itest_helper.py'
--- lib/canonical/launchpad/mailman/itests/itest_helper.py	2008-02-22 16:29:30 +0000
+++ lib/canonical/launchpad/mailman/itests/itest_helper.py	2008-02-22 21:58:35 +0000
@@ -8,6 +8,7 @@
 import time
 import errno
 import base64
+import shutil
 import signal
 import socket
 import mailbox
@@ -17,16 +18,21 @@
 from email import message_from_file
 from subprocess import Popen, PIPE
 
+
 __all__ = [
     'HERE',
     'IntegrationTestFailure',
     'MAILMAN_BIN',
     'TOP',
+    'create_list',
     'create_transaction_manager',
+    'dump_list_info',
     'make_browser',
     'num_requests_pending',
+    'prepare_for_sync'
     'review_list',
     'run_mailman',
+    'subscribe',
     'transactionmgr',
     ]
 
@@ -280,3 +286,131 @@
         return mailing_list.NumRequestsPending()
     finally:
         mailing_list.Unlock()
+
+
+def create_list(team_name):
+    """Do everything you need to do to make the team's list live."""
+    displayname = ' '.join(word.capitalize() for word in team_name.split('-'))
+    browser = make_browser()
+    # Create the team.
+    browser.open('http://launchpad.dev/people/+newteam')
+    browser.getControl(name='field.name').value = team_name
+    browser.getControl('Display Name').value = displayname
+    browser.getControl(name='field.subscriptionpolicy').displayValue = [
+        'Open Team']
+    browser.getControl('Create').click()
+    # Create the mailing list.
+    beta_program_enable(team_name)
+    browser.reload()
+    browser.getLink('Configure mailing list').click()
+    browser.getControl('Apply for Mailing List').click()
+    review_list(team_name)
+    from Mailman.Utils import list_names
+    assert team_name in list_names(), (
+        'Mailing list was not created: %s' % list_names())
+
+
+def subscribe(first_name, team_name):
+    """Do everything you need to subscribe a person to a mailing list."""
+    from canonical.database.sqlbase import commit
+    from canonical.launchpad.ftests import login, logout
+    from canonical.launchpad.ftests.mailinglists_helper import new_person
+    from canonical.launchpad.interfaces import IMailingListSet, IPersonSet
+    from zope.component import getUtility
+    # Create the person if she does not already exist, and join her to the
+    # team.
+    login('foo.bar@canonical.com')
+    person_set = getUtility(IPersonSet)
+    person = person_set.getByName(first_name.lower())
+    if person is None:
+        person = new_person(first_name)
+    team = getUtility(IPersonSet).getByName(team_name)
+    person.join(team)
+    # Subscriber her to the list.
+    mailing_list = getUtility(IMailingListSet).get(team_name)
+    mailing_list.subscribe(person)
+    logout()
+    serial_watcher = LogWatcher('serial')
+    commit()
+    serial_watcher.wait()
+
+
+def prepare_for_sync():
+    """Prepare a sync'd directory for mlist-sync.py."""
+    from canonical.config import config
+    from canonical.database.sqlbase import commit
+    from canonical.launchpad.ftests import login, logout
+    from canonical.launchpad.interfaces import IEmailAddressSet
+    from zope.component import getUtility
+    # Tweak each of the mailing lists by essentially breaking their host_name
+    # and web_page_urls.  These will get repaired by the sync script.  Do this
+    # before we copy so that the production copy will have the busted values.
+    from Mailman import mm_cfg
+    from Mailman.MailList import MailList
+    from Mailman.Utils import list_names
+    team_names = list_names()
+    for list_name in team_names:
+        if list_name == mm_cfg.MAILMAN_SITE_LIST:
+            continue
+        mailing_list = MailList(list_name)
+        try:
+            mailing_list.host_name = 'lists.prod.launchpad.dev'
+            mailing_list.web_page_url = 'http://lists.prod.launchpad.dev'
+            mailing_list.Save()
+        finally:
+            mailing_list.Unlock()
+    # Create a mailing list that exists only in Mailman.  The sync script will
+    # end up deleting this because it represents a race condition between when
+    # the production database was copied and when the Mailman data was copied.
+    mlist = MailList()
+    try:
+        mlist.Create('fake-team', mm_cfg.SITE_LIST_OWNER, ' no password ')
+        mlist.Save()
+        os.makedirs(os.path.join(mm_cfg.VAR_PREFIX, 'mhonarc', 'fake-team'))
+    finally:
+        mlist.Unlock()
+    # Calculate a directory in which to put the simulated production database,
+    # then copy our current Mailman stuff to it, lock, stock, and barrel.
+    tempdir = tempfile.mkdtemp()
+    source_dir = os.path.join(tempdir, 'production')
+    shutil.copytree(config.mailman.build.var_dir, source_dir, symlinks=True)
+    # Now, we have to mess up the production database by tweaking the email
+    # addresses of all the mailing lists.
+    login('foo.bar@canonical.com')
+    email_set = getUtility(IEmailAddressSet)
+    for list_name in team_names:
+        if list_name == mm_cfg.MAILMAN_SITE_LIST:
+            continue
+        email = email_set.getByEmail(list_name + '@lists.launchpad.dev')
+        email.email = list_name + '@lists.prod.launchpad.dev'
+    logout()
+    commit()
+    return source_dir
+
+
+def dump_list_info():
+    """Print a bunch of useful information related to sync'ing."""
+    from canonical.database.sqlbase import flush_database_caches
+    from canonical.launchpad.ftests import login, logout
+    from canonical.launchpad.interfaces import IEmailAddressSet, IPersonSet
+    from zope.component import getUtility
+    from Mailman import mm_cfg
+    from Mailman.MailList import MailList
+    from Mailman.Utils import list_names
+    # Print interesting information about each mailing list.
+    flush_database_caches()
+    login('foo.bar@canonical.com')
+    for list_name in sorted(list_names()):
+        if list_name == mm_cfg.MAILMAN_SITE_LIST:
+            continue
+        mailing_list = MailList(list_name, lock=False)
+        print mailing_list.internal_name()
+        print '   ', mailing_list.host_name, mailing_list.web_page_url
+        team = getUtility(IPersonSet).getByName(list_name)
+        if team is None:
+            print '    No Launchpad team:', list_name
+        else:
+            mlist_addresses = getUtility(IEmailAddressSet).getByPerson(team)
+            for email in sorted(email.email for email in mlist_addresses):
+                print '   ', email
+    logout()

=== modified file 'lib/canonical/launchpad/mailman/itests/runtests.py'
--- lib/canonical/launchpad/mailman/itests/runtests.py	2008-01-22 20:35:09 +0000
+++ lib/canonical/launchpad/mailman/itests/runtests.py	2008-02-22 21:58:35 +0000
@@ -91,7 +91,7 @@
     # Now delete any mailing lists still hanging around.  We don't care if
     # this fails because it means the list doesn't exist.  While we're at it,
     # remove any related archived backup files.
-    for team_name in ('itest-one', 'itest-two', 'itest-three'):
+    for team_name in ('itest-one', 'itest-two', 'itest-three', 'fake-team'):
         try:
             itest_helper.run_mailman('./rmlist', '-a', team_name)
         except itest_helper.IntegrationTestFailure:

=== added file 'lib/canonical/launchpad/mailman/itests/staging.txt'
--- lib/canonical/launchpad/mailman/itests/staging.txt	1970-01-01 00:00:00 +0000
+++ lib/canonical/launchpad/mailman/itests/staging.txt	2008-02-22 21:58:35 +0000
@@ -0,0 +1,160 @@
+= Staging migration =
+
+Every once in a while we copy production's database over to staging so that
+testing can be done there on the real database, but without any chance of
+messing up the real site.  When this happens, we also need to synchronize
+production's Mailman directories so that the mailing lists all stay consistent
+(well, within the limits of the inherent race conditions).  To do this, we use
+scripts/mlist-sync.py.
+
+
+== Synchronization ==
+
+It's actually fairly difficult to recreate what happens during the staging
+sync, but we can get close.  Start off by creating a few mailing lists and
+sending some messages to them.
+
+    >>> import itest_helper
+    >>> itest_helper.create_list('itest-one')
+    >>> itest_helper.create_list('itest-two')
+    >>> itest_helper.subscribe('Anne', 'itest-one')
+    >>> itest_helper.subscribe('Anne', 'itest-two')
+    >>> itest_helper.subscribe('Bart', 'itest-one')
+    >>> itest_helper.subscribe('Cris', 'itest-two')
+
+    >>> smtpd = itest_helper.SMTPServer()
+    >>> smtpd.start()
+    >>> smtp_watcher = itest_helper.LogWatcher('smtp')
+    >>> from Mailman.Post import inject
+    >>> inject('itest-one', """\
+    ... From: anne.person@example.com
+    ... To: itest-one@lists.launchpad.dev
+    ... Subject: hello
+    ... Message-ID: <first-injection>
+    ...
+    ... Hello team!
+    ... """)
+    >>> smtp_watcher.wait()
+
+    >>> inject('itest-two', """\
+    ... From: anne.person@example.com
+    ... To: itest-two@lists.launchpad.dev
+    ... Subject: hello
+    ... Message-ID: <second-injection>
+    ...
+    ... Hello team!
+    ... """)
+    >>> smtp_watcher.wait()
+
+    >>> inject('itest-one', """\
+    ... From: bart.person@example.com
+    ... To: itest-one@lists.launchpad.dev
+    ... Subject: hello
+    ... Message-ID: <third-injection>
+    ...
+    ... Hello team!
+    ... """)
+    >>> smtp_watcher.wait()
+
+    >>> inject('itest-two', """\
+    ... From: cris.person@example.com
+    ... To: itest-two@lists.launchpad.dev
+    ... Subject: hello
+    ... Message-ID: <fourth-injection>
+    ...
+    ... Hello team!
+    ... """)
+    >>> smtp_watcher.wait()
+
+    >>> from operator import itemgetter
+    >>> for message in sorted(smtpd.getMessages(), key=itemgetter('sender')):
+    ...     print message['sender']
+    itest-one-bounces+anne.person=example.com@lists.launchpad.dev
+    itest-one-bounces+anne.person=example.com@lists.launchpad.dev
+    itest-one-bounces+archive=mail-archive.com@lists.launchpad.dev
+    itest-one-bounces+archive=mail-archive.com@lists.launchpad.dev
+    itest-one-bounces+bart.person=example.com@lists.launchpad.dev
+    itest-one-bounces+bart.person=example.com@lists.launchpad.dev
+    itest-two-bounces+anne.person=example.com@lists.launchpad.dev
+    itest-two-bounces+anne.person=example.com@lists.launchpad.dev
+    itest-two-bounces+archive=mail-archive.com@lists.launchpad.dev
+    itest-two-bounces+archive=mail-archive.com@lists.launchpad.dev
+    itest-two-bounces+cris.person=example.com@lists.launchpad.dev
+    itest-two-bounces+cris.person=example.com@lists.launchpad.dev
+
+Give one of the mailing lists a contact address, to further simulate what the
+sync script has to deal with.
+
+    >>> from canonical.database.sqlbase import commit
+    >>> from canonical.launchpad.ftests import login, logout
+    >>> from canonical.launchpad.interfaces import (
+    ...     EmailAddressStatus, IEmailAddressSet, IPersonSet)
+    >>> from zope.component import getUtility
+    >>> login('foo.bar@canonical.com')
+    >>> team = getUtility(IPersonSet).getByName('itest-two')
+    >>> email = getUtility(IEmailAddressSet).new(
+    ...     'contact@example.com', team, EmailAddressStatus.VALIDATED)
+    >>> team.setContactAddress(email)
+    >>> logout()
+    >>> commit()
+
+Now we need to simulate the difference between the staging and production
+databases.  To do this, we'll use a helper, but first we need to stop Mailman.
+
+    >>> from canonical.launchpad.mailman import runmailman
+    >>> runmailman.stop_mailman(quiet=True)
+    >>> source_dir = itest_helper.prepare_for_sync()
+
+Here's what these lists look like before they're sync'd.
+
+    >>> itest_helper.dump_list_info()
+    fake-team
+        lists.launchpad.dev http://lists.launchpad.dev/mailman/
+        No Launchpad team: fake-team
+    itest-one
+        lists.prod.launchpad.dev http://lists.prod.launchpad.dev/
+        itest-one@lists.prod.launchpad.dev
+    itest-two
+        lists.prod.launchpad.dev http://lists.prod.launchpad.dev/
+        contact@example.com
+        itest-two@lists.prod.launchpad.dev
+
+Do the sync by calling the script.  We must set the LPCONFIG environment
+variable for the script to find the correct launchpad.conf file.
+
+    >>> from canonical.config import config
+    >>> from subprocess import *
+    >>> proc = Popen(
+    ...     ('scripts/mlist-sync.py', '--hostname',
+    ...      'lists.prod.launchpad.dev', source_dir),
+    ...     stdout=PIPE, stderr=PIPE, cwd=config.root,
+    ...     env=dict(LPCONFIG='mailman-itests'))
+    >>> stdout, stderr = proc.communicate()
+
+    # Ignore stdout and stderr, but print the return code.  stdout should be
+    # completely empty and stderr really is uninteresting.  It's also
+    # problematic because of the way log files are printed (breaking doctest
+    # ellipses).
+    >>> proc.returncode
+    0
+
+If the script ran correctly, Mailman should only know about the two mailing
+lists we created above, and not the fake-team list.  Also, the host_name and
+web_page_url attributes for these lists should not point to 'prod'.
+
+    >>> itest_helper.dump_list_info()
+    itest-one
+        lists.launchpad.dev http://lists.launchpad.dev/mailman/
+        itest-one@lists.launchpad.dev
+    itest-two
+        lists.launchpad.dev http://lists.launchpad.dev/mailman/
+        contact@example.com
+        itest-two@lists.launchpad.dev
+
+
+== Clean up ==
+
+    >>> import shutil
+    >>> shutil.rmtree(source_dir)
+    >>> runmailman.start_mailman(quiet=True)
+    >>> smtpd.stop()

=== modified file 'lib/canonical/launchpad/mailman/runmailman.py'
--- lib/canonical/launchpad/mailman/runmailman.py	2007-10-23 15:56:18 +0000
+++ lib/canonical/launchpad/mailman/runmailman.py	2008-02-22 21:58:35 +0000
@@ -17,16 +17,19 @@
 from canonical.launchpad.mailman.monkeypatches import monkey_patch
 
 
-def stop_mailman():
+def stop_mailman(quiet=False):
     """Stop the Mailman master qrunner, which kills all its subprocesses."""
     mailman_path = config.mailman.build.prefix
     mailman_bin  = os.path.join(mailman_path, 'bin')
-    code = subprocess.call(('./mailmanctl', 'stop'), cwd=mailman_bin)
+    args = ['./mailmanctl', 'stop']
+    if quiet:
+        args.insert(1, '--quiet')
+    code = subprocess.call(args, cwd=mailman_bin)
     if code:
         print >> sys.stderr, 'mailmanctl did not stop cleanly:', code
 
 
-def start_mailman():
+def start_mailman(quiet=False):
     """Start the Mailman master qrunner.
 
     The client of start_mailman() is responsible for ensuring that
@@ -41,7 +44,10 @@
     # Monkey-patch the installed Mailman 2.1 tree.
     monkey_patch(mailman_path, config)
     # Start the Mailman master qrunner.
-    code = subprocess.call(('./mailmanctl', 'start'), cwd=mailman_bin)
+    args = ['./mailmanctl', 'start']
+    if quiet:
+        args.insert(1, '--quiet')
+    code = subprocess.call(args, cwd=mailman_bin)
     if code:
         print >> sys.stderr, 'mailmanctl did not start cleanly'
         sys.exit(code)

=== modified file 'scripts/mlist-sync.py'
--- scripts/mlist-sync.py	2008-02-20 18:28:53 +0000
+++ scripts/mlist-sync.py	2008-02-22 21:58:35 +0000
@@ -153,7 +153,9 @@
                 self.deleteMailmanList(list_name)
                 continue
 
-            # Teams with mailing lists should have only one email address.
+            # Clean up the team email addresses corresponding to their mailing
+            # lists.  Note that teams can have two email addresses if they
+            # have a different contact address.
             team = getUtility(IPersonSet).getByName(list_name)
             mlist_addresses = email_address_set.getByPerson(team)
             if mlist_addresses.count() == 0:

